use bevy_ecs::prelude::Resource;
use bevy_ecs::world::World;
use std::sync::Arc;
use vulkano::{
    command_buffer::allocator::StandardCommandBufferAllocator,
    descriptor_set::allocator::StandardDescriptorSetAllocator,
    device::{Device, Queue},
    instance::Instance,
    memory::allocator::StandardMemoryAllocator,
};

/// Vulkan Instance resource
#[derive(Resource)]
pub struct VulkanInstance(pub Arc<Instance>);

/// Vulkan Device resource
#[derive(Resource)]
pub struct VulkanDevice(pub Arc<Device>);

/// Vulkan Graphics Queue resource
#[derive(Resource)]
pub struct VulkanGraphicsQueue(pub Arc<Queue>);

/// Vulkan Compute Queue resource
#[derive(Resource)]
pub struct VulkanComputeQueue(pub Arc<Queue>);

/// Vulkan Transfer Queue resource
#[derive(Resource)]
pub struct VulkanTransferQueue(pub Option<Arc<Queue>>);

/// Vulkan Memory Allocator resource
#[derive(Resource)]
pub struct VulkanMemoryAllocator(pub Arc<StandardMemoryAllocator>);

/// Vulkan Command Buffer Allocator resource
#[derive(Resource)]
pub struct VulkanCommandBufferAllocator(pub Arc<StandardCommandBufferAllocator>);

/// Vulkan Descriptor Set Allocator resource
#[derive(Resource)]
pub struct VulkanDescriptorSetAllocator(pub Arc<StandardDescriptorSetAllocator>);

/// Helper functions to access vulkan resources from the ECS world
impl VulkanDevice {
    pub fn get_from_world(world: &World) -> &Arc<Device> {
        &world.resource::<VulkanDevice>().0
    }
}

impl VulkanMemoryAllocator {
    pub fn get_from_world(world: &World) -> &Arc<StandardMemoryAllocator> {
        &world.resource::<VulkanMemoryAllocator>().0
    }
}

impl VulkanCommandBufferAllocator {
    pub fn get_from_world(world: &World) -> &Arc<StandardCommandBufferAllocator> {
        &world.resource::<VulkanCommandBufferAllocator>().0
    }
}

impl VulkanDescriptorSetAllocator {
    pub fn get_from_world(world: &World) -> &Arc<StandardDescriptorSetAllocator> {
        &world.resource::<VulkanDescriptorSetAllocator>().0
    }
}

impl VulkanGraphicsQueue {
    pub fn get_from_world(world: &World) -> &Arc<Queue> {
        &world.resource::<VulkanGraphicsQueue>().0
    }
}

impl VulkanTransferQueue {
    pub fn get_from_world(world: &World) -> Option<&Arc<Queue>> {
        world.resource::<VulkanTransferQueue>().0.as_ref()
    }
}
